const ID_TOOLBAR = "#toolbar";
const ID_SELECTEDITEM = "#selected-item";
const ID_SELECTED_PARAMS = "#selected-params";
const ID_SELECTED_SOURCES = "#selected-sources";
const ID_SELECTED_TARGETS = "#selected-targets";
const ID_SELECTEDITEMS = "#selected-items";

const DOM_LIST_ITEM = "li";

const TOOL_ICON_SRC = PLUGIN_PATH + "editor/images/tools/";
const TOOL_ICON_FORMAT = ".png";
const TOOL_SELECTED_CLASS = "selected";

class Display {
    constructor(tools) {
        this.tools = Object.values(tools);
        this.previousTool = undefined;

        this.renderToolbar(this.tools);
    }

    setSelectedTool(tool) {
        var selectedTool = jQuery(Display.getToolId(tool));
        selectedTool.addClass(TOOL_SELECTED_CLASS);

        if (this.previousTool !== undefined) {
            var previousTool = jQuery(Display.getToolId(this.previousTool));
            previousTool.removeClass(TOOL_SELECTED_CLASS);
        }

        this.previousTool = tool;
    }

    renderToolbar(tools) {
        this.fillDomList(ID_TOOLBAR, tools, this.toolRenderer);
    }

    static getToolId(tool) {
        return ID_TOOLBAR + "-" + tool.getKey();
    }

    toolRenderer(tool) {
        return (
            '<button id="' +
            Display.getToolId(tool).substr(1) + // Remove # from id
            '"onclick="state.setTool(TOOLS.' +
            tool.getKey() +
            ')" title="' +
            tool.getName() +
            '"><img src="' +
            TOOL_ICON_SRC +
            tool.getIcon() +
            TOOL_ICON_FORMAT +
            '"></button>'
        );
    }

    setSelectedItem(item) {
        jQuery(ID_SELECTEDITEM).html(Display.toStr(item));

        var paramsDOM = jQuery(ID_SELECTED_PARAMS);
        paramsDOM.empty();

        var params = NODE_PARAMS;
        if (item === undefined) {
            params = [];
        } else if (item.link) {
            params = LINK_PARAMS;
        }

        params.forEach((param) => {
            paramsDOM.append(
                "<" +
                    DOM_LIST_ITEM +
                    ">" +
                    param +
                    ' <textarea>' +
                    (item[param] === undefined ? "" : item[param]) +
                    '</textarea></' +
                    DOM_LIST_ITEM +
                    ">"
            );
        });

        // Render Source and Target list
        var sources = [];
        var targets = [];
        if (item !== undefined && item.node) {
            var nodes = graph.data[GRAPH_NODES];
            for (var i = 0; i < nodes.length; i++) {
                if (graph.existsLink(nodes[i][NODE_ID], item[NODE_ID])) {
                    sources.push(nodes[i]);
                } else if (graph.existsLink(item[NODE_ID], nodes[i][NODE_ID])) {
                    targets.push(nodes[i]);
                }
            }
        } else if (item !== undefined && item.link) {
            sources.push(item[LINK_SOURCE]);
            targets.push(item[LINK_TARGET]);
        }

        this.fillDomList(ID_SELECTED_SOURCES, sources, this.graphItemRenderer);
        this.fillDomList(ID_SELECTED_TARGETS, targets, this.graphItemRenderer);
    }

    setSelectedItems(items, itemsContext) {
        this.fillDomList(ID_SELECTEDITEMS, items, this.graphItemRenderer);
    }

    graphItemRenderer(item) {
        return (
            "<" +
            DOM_LIST_ITEM +
            ">" +
            Display.toStr(item) +
            "</" +
            DOM_LIST_ITEM +
            ">"
        );
    }

    fillDomList(listId, items, itemRenderer) {
        var listCont = jQuery(listId);
        listCont.empty();

        items.forEach((i) => listCont.append(itemRenderer(i)));
    }

    static toStr(item) {
        if (item === undefined) {
            return "UNDEFINED";
        }

        if (item.node) {
            return item[NODE_LABEL] + " [" + item[NODE_ID] + "]";
        } else if (item.link) {
            return (
                Display.toStr(item[LINK_SOURCE]) +
                " <-> " +
                Display.toStr(item[LINK_TARGET])
            );
        } else {
            return "UNDEFINED";
        }
    }
}
